@@ -691,11 +691,12 @@ def normalize_sparse(x, y, v, resolution, distance_in_px):
     n = max(max(x),max(y)) + 1
 
     #distance_in_px = min(distance_in_px, n)
+    pval_weights = []
     distances = np.abs(y-x)
     if (n-distance_in_px)*resolution  > 2000000: 
         with warnings.catch_warnings():
             warnings.simplefilter('ignore', category=RuntimeWarning)
-            filter_size = int(2000000 / resolution)
+            filter_size = int(2000000 / resolution)             
             for d in range(2 + distance_in_px):
                 indices = distances == d
                 vals = np.zeros(n-d)
@@ -732,7 +733,7 @@ def normalize_sparse(x, y, v, resolution, distance_in_px):
                 vals[x[indices]] /= local_std[x[indices]]
                 np.nan_to_num(vals, copy=False, nan=0, posinf=0, neginf=0)
                 vals = vals*(1 + math.log(1+mean, 30))
-            
+                pval_weights += [1 + math.log(1+mean, 30)]
                 v[indices] = vals[x[indices]]
     else:
         with warnings.catch_warnings():
@@ -750,6 +751,7 @@ def normalize_sparse(x, y, v, resolution, distance_in_px):
             #print(std)
                 v[indices] = (v[indices] - mean)/std
                 np.nan_to_num(v, copy=False, nan=0, posinf=0, neginf=0) 
+    return pval_weights
 			
 def inter_nrmalize_map(vals):
     m = np.mean(vals)
@@ -759,9 +761,10 @@ def inter_nrmalize_map(vals):
     np.nan_to_num(cmap, copy=False, nan=0, posinf=0, neginf=0)
 
 
-def mustache(c, chromosome,chromosome2, res, start, end, mask_size, distance_in_px, octave_values, st, pt):
+def mustache(c, chromosome,chromosome2, res, pval_weights, start, end, mask_size, distance_in_px, octave_values, st, pt):
 
     nz = np.logical_and(c != 0, np.triu(c, 4))
+    nz_temp = np.logical_and.reduce((c != 0, np.triu(c, 4)>0, np.tril(c, distance_in_px)>0))
     if np.sum(nz) < 50:
         return []
     c[np.tril_indices_from(c, 4)] = 2
@@ -838,9 +841,18 @@ def mustache(c, chromosome,chromosome2, res, start, end, mask_size, distance_in_
     pFound = pAll != 2
     if len(pFound) < 10000:
         return []
+    
     _, pCorrect, _, _ = multipletests(pAll[pFound], method='fdr_bh')
     pAll[pFound] = pCorrect
 
+    #################
+    #o = np.ones_like(c)
+    #o[nz] = pAll
+    #x, y = np.where(nz_temp)
+    #o[x,y]*=np.array(pval_weights)[y-x]
+    #o[x,y]/=10
+    #pAll = o[nz]
+    #################
     o = np.ones_like(c)
     o[nz] = pAll
     sig_count = np.sum(o < pt) #change
@@ -948,7 +960,7 @@ def regulator(f, norm_method, CHRM_SIZE, outdir, bed="",
     distance_in_px = int(math.ceil(distance_in_bp // res))
     if chromosome == chromosome2:
         n = max(max(x), max(y)) + 1
-        normalize_sparse(x, y, v, res, distance_in_px)
+        pval_weights = normalize_sparse(x, y, v, res, distance_in_px)
         CHUNK_SIZE = max(2*distance_in_px, 2000)
         overlap_size = distance_in_px
 
@@ -964,7 +976,7 @@ def regulator(f, norm_method, CHRM_SIZE, outdir, bed="",
                 end.append(start[-1] + CHUNK_SIZE)
             end[-1] = n
             start[-1] = end[-1] - CHUNK_SIZE
-
+        
         print("Loop calling...")
         with Manager() as manager:
             o = manager.list()
@@ -980,13 +992,15 @@ def regulator(f, norm_method, CHRM_SIZE, outdir, bed="",
                 cc[xc,yc] = vc
                 #             
                 p = Process(target=process_block, args=(
-                    i, start, end, overlap_size, cc, chromosome,chromosome2, res, distance_in_px, octave_values, o, st, pt))
+                    i, start, end, overlap_size, cc, chromosome,chromosome2, res, pval_weights, distance_in_px, octave_values, o, st, pt))
                 p.start()
                 processes.append(p)
                 if len(processes) >= nprocesses or i == (len(start) - 1):
                     for p in processes:
                         p.join()
                     processes = []
+            #o_corrected = [[e[0],e[1],e[2]/pval_weights[e[1]-e[0]],e[3]] for e in list(o)]
+                
             return list(o)
         
     else:
@@ -996,7 +1010,7 @@ def regulator(f, norm_method, CHRM_SIZE, outdir, bed="",
 	
  
 
-def process_block(i, start, end, overlap_size, cc, chromosome,chromosome2, res, distance_in_px, octave_values, o, st, pt):
+def process_block(i, start, end, overlap_size, cc, chromosome,chromosome2, res, pval_weights, distance_in_px, octave_values, o, st, pt):
     print("Starting block ", i+1, "/", len(start), "...", sep='')
     if i == 0:
         mask_size = -1
@@ -1005,7 +1019,7 @@ def process_block(i, start, end, overlap_size, cc, chromosome,chromosome2, res,
     else:
         mask_size = overlap_size
     loops = mustache(
-        cc, chromosome,chromosome2, res, start[i], end[i], mask_size, distance_in_px, octave_values, st, pt)
+        cc, chromosome,chromosome2, res, pval_weights, start[i], end[i], mask_size, distance_in_px, octave_values, st, pt)
     for loop in list(loops):
         if loop[0] >= start[i]+mask_size or loop[1] >= start[i]+mask_size:
             o.append([loop[0], loop[1], loop[2], loop[3]])
@@ -1069,6 +1083,7 @@ def main():
         distFilter = 2000000
         print("The distance limit is set to 2Mbp")
 
+    #distFilter = 4000000
     chrSize_in_bp = False
     if CHR_COOL_FLAG:
         # extract all the chromosome names big enough to run mustache on
